package com.loafle.overflow.probe.container.crawler.impl.database;

import com.loafle.overflow.probe.container.crawler.Crawler;
import com.loafle.overflow.probe.container.crawler.config.Config;
import com.loafle.overflow.probe.container.crawler.config.Item;
import com.loafle.overflow.probe.container.crawler.result.ResultSet;
import com.loafle.overflow.probe.container.crawler.result.ResultSetRow;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

public abstract class DatabaseCrawler extends Crawler {

    public DatabaseCrawler() {
    }

    @Override
    public List<ResultSet> getInternal(Config config) throws Exception {
        return getMetrics(config);
    }

    private List<ResultSet> getMetrics(Config config) throws Exception {

        Connection conn = null;
        Statement stmt = null;
        java.sql.ResultSet rs = null;

        String url = (String)config.getTarget().getAuth().get("url");
        String id =  (String)config.getTarget().getAuth().get("id");
        String pw =  (String)config.getTarget().getAuth().get("pw");

        loadDriverClass();

        List<ResultSet> resultSets = new ArrayList<>();

        try {
            conn = java.sql.DriverManager.getConnection(url, id, pw);
            stmt = conn.createStatement();

            for (Item item : config.getItems()) {

                ResultSet resultSet = ResultSetRow.newInstance(item);
                Map<String,Integer> meta = resultSet.getMeta();

                rs = stmt.executeQuery((String)item.getQueryInfo().getQuery());

                while(rs.next()) {
                    List<String> row =  new ArrayList<>(Arrays.asList(new String[meta.size()]));
                    for (Map.Entry<String, Integer> info : meta.entrySet()) {
                        String data = rs.getString(info.getKey());
                        data = data.trim();
                        row.set(info.getValue().intValue(),data);
                    }
                    resultSet.addRow(row);
                }

                resultSets.add(resultSet);
                rs.close();
            }
            return resultSets;

        } catch (Exception e) {
            throw e;
        } finally {
            if (conn != null) {try {conn.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
            if (stmt != null) {try {stmt.close();} catch (SQLException e) {e.printStackTrace();}}
        }
    }

    protected abstract void loadDriverClass() throws ClassNotFoundException;

}
