package com.loafle.overflow.probe.container.crawler.impl.redis;

import com.loafle.overflow.probe.container.crawler.Crawler;
import com.loafle.overflow.probe.container.crawler.config.Config;
import com.loafle.overflow.probe.container.crawler.config.Item;
import com.loafle.overflow.probe.container.crawler.result.ResultSet;
import redis.clients.jedis.Jedis;

import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RedisCralwer extends Crawler {

    private static final Logger LOGGER = Logger.getLogger(RedisCralwer.class.getName());

    public RedisCralwer() {

    }

    public String name() {
        return "REDIS_CRAWLER";
    }

    @Override
    public List<ResultSet> getInternal(Config c) throws Exception {
        LOGGER.log(Level.INFO ,"getInternal call");
        return collectMetric(c);
    }

    protected List<ResultSet> collectMetric(Config c) {

        String targetIP = c.getTarget().getConnection().getIp();
        int targetPort = Integer.valueOf(c.getTarget().getConnection().getPort());

        String authpw = (String)c.getTarget().getAuth().get("authpw");

        Jedis jedis = null;
        List<Item> items = c.getItems();

        try {
            jedis = new Jedis(targetIP, targetPort,false);
            if (authpw != null && !authpw.equals("")) {
                String code = jedis.auth(authpw);
                if (!code.equals("")) {
                    // set error auth
                }
            }

            List<ResultSet> resultSetList = new ArrayList<>();
            for (Item item : items) {
                ResultSet resultSet = ResultSet.newInstance(item);

                String queryString = (String) item.getQueryInfo().getQuery();
                String info = jedis.info(queryString);
                Map<String, String> resultMap = (Map<String, String>) parseToMap(info).get(queryString);

                resultSet.addRow(parse(resultMap,resultSet.getMeta()));
                resultSetList.add(resultSet);
            }

            return resultSetList;
        } catch (Exception e) {
            throw e;
        } finally {
            if (jedis != null) {
                jedis.close();
            }
        }

    }

    private List<String> parse(Map<String, String> dataMap, Map<String, Integer> meta) {

        List<String> row =  new ArrayList<>(Arrays.asList(new String[meta.size()]));
        for (Map.Entry<String, Integer> info : meta.entrySet()) {
            String data = dataMap.get(info.getKey());
            data = data.trim();
            row.set(info.getValue().intValue(),data);
        }
        return row;
    }

    public Map<String, Object> parseToMap(String source) {
        String[] categorys = source.split("\r\n\r\n");
        Map<String, Object> result = new HashMap<String, Object>();

        for (String category : categorys) {

            if (category.length() == 0) {continue;}
            Map<String, String> sm = new HashMap<String,String>();
            String[] lines = category.split("\r\n");

            int idx = 0;
            for (String line : lines) {

                if (idx == 0) {idx++;continue;}
                if (line.length() == 0) {continue;}

                String[] items = line.split(":");
                if (items.length != 0 && items.length > 1) {
                    sm.put(items[0], items[1]);
                }
            }
            String[] ca = lines[0].split(" ");
            result.put(ca[1], sm);
        }
        return result;
    }
}
