package com.loafle.overflow.probe.container.crawler.impl.snmp.version;

import org.snmp4j.CommunityTarget;
import org.snmp4j.PDU;
import org.snmp4j.Snmp;
import org.snmp4j.mp.SnmpConstants;
import org.snmp4j.smi.*;
import org.snmp4j.util.DefaultPDUFactory;
import org.snmp4j.util.TreeUtils;

import java.util.Map;

public class SNMPv2c extends SNMP {

    private Snmp snmp =  null;

    public SNMPv2c(Snmp snmp) {
        this.snmp = snmp;
    }

    private CommunityTarget getTarget(String addr, String port, String community) {
        UdpAddress address = new UdpAddress(addr + "/" + port);
        CommunityTarget target = new CommunityTarget();
        target.setCommunity(new OctetString(community));
        target.setAddress(address);
        target.setRetries(2);
        target.setTimeout(5000);
        target.setVersion(SnmpConstants.version2c);
        return target;
    }

    public Boolean validate(String addr, String port, String community) throws Exception {
        String testOid = "1.3.6.1.2.1.1.3.0";
        Map<String, String> result = this.get(addr, port, community, new String[]{testOid});
        if (result.get(testOid) != null) return true;
        return false;
    }

    public Map<String, String> walk(String addr, String port, String community, String oidStr) throws Exception  {

        TreeUtils treeUtils = new TreeUtils(this.snmp, new DefaultPDUFactory());
        CommunityTarget target = getTarget(addr, port, community);

        return super.walk(oidStr, target, treeUtils);
    }


    public Map<String, String> get(String addr, String port, String community, String[] oids) throws Exception {

        CommunityTarget target = getTarget(addr, port, community);

        PDU pdu = new PDU();
        for (String oid : oids) {
            pdu.add(new VariableBinding(new OID(oid)));
        }
        pdu.setType(PDU.GET);
        pdu.setRequestID(new Integer32(1));

        return super.get(this.snmp, pdu, target);
    }

}

