package com.loafle.overflow.probe.container.server;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.loafle.overflow.probe.container.crawler.Crawler;
import com.loafle.overflow.probe.container.crawler.impl.database.mysql.MySQLCrawler;
import com.loafle.overflow.probe.container.crawler.impl.database.oracle.OracleCrawler;
import com.loafle.overflow.probe.container.crawler.impl.database.postgresql.PostgreSQLCrawler;
import com.loafle.overflow.probe.container.crawler.impl.database.sqlserver.SQLServerCrawler;
import com.loafle.overflow.probe.container.crawler.impl.jmx.JMXCrawler;
import com.loafle.overflow.probe.container.crawler.impl.mongodb.MongoDBCrawler;
import com.loafle.overflow.probe.container.crawler.impl.redis.RedisCralwer;
import com.loafle.overflow.probe.container.crawler.impl.snmp.SNMPCrawler;
import com.loafle.overflow.probe.container.crawler.impl.wmi.WMICrawler;
import com.loafle.overflow.probe.container.rpc.invoker.Invoker;
import com.loafle.overflow.probe.container.rpc.invoker.impl.SimpleInvoker;
import com.loafle.overflow.probe.container.service.ConfigService;
import com.loafle.overflow.probe.container.service.DataService;
import com.loafle.overflow.probe.container.service.StateService;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.ChannelInitializer;

import java.util.HashMap;
import java.util.Map;

public abstract class ContainerConfiguration {
    private static ObjectMapper objectMapper;
    private static ContainerChannelInitializer channelInitializer;
    private static ChannelInboundHandlerAdapter channelInboundHandlerAdapter;
    private static Invoker invoker;
    private static Map<String, Crawler> crawlers = new HashMap<>();
    private static Map<String, Object> services = new HashMap<>();

    static {
        addCrawler(new MySQLCrawler());
        addCrawler(new OracleCrawler());
        addCrawler(new PostgreSQLCrawler());
        addCrawler(new SQLServerCrawler());
        addCrawler(new JMXCrawler());
        addCrawler(new MongoDBCrawler());
        addCrawler(new RedisCralwer());
        addCrawler(new SNMPCrawler());
        addCrawler(new WMICrawler());

        addService(new ConfigService(crawlers));
        addService(new DataService(crawlers));
        addService(new StateService(crawlers));
    }

    private static void addCrawler(Crawler crawler) {
        String name = crawler.name();
        crawlers.put(name, crawler);
    }

    private static void addService(Object service) {
        String name = service.getClass().getSimpleName();
        services.put(name, service);
    }

    public static ObjectMapper objectMapper() {
        if (null == objectMapper) {
            objectMapper = new ObjectMapper();
        }
        return objectMapper;
    }

    public static ChannelInitializer channelInitializer() {
        if (null == channelInitializer) {
            channelInitializer = new ContainerChannelInitializer();
        }
        return channelInitializer;
    }

    public static ChannelInboundHandlerAdapter channelInboundHandlerAdapter() {
        if (null == channelInboundHandlerAdapter) {
            channelInboundHandlerAdapter = new ContainerJSONHandler();
        }
        return channelInboundHandlerAdapter;
    }

    public static Invoker invoker() {
        if (null == invoker) {
            invoker = new SimpleInvoker(objectMapper(), services);
        }
        return invoker;
    }
}
